##############################################################################
# Module : setdestdlg.py
# Author : Old King Cole
# Date   : 15 October 2005
#
# Description : Dialog for setting the download destination of a torrent
#
##############################################################################
import sys, wx
from os import path
from string import whitespace

from BitTornado.bencode import bdecode

from Dialogs.addsamedestdlg import AddSameDestDialog

sysencoding = sys.getfilesystemencoding()


class SetDestDialog(wx.Dialog):
    def __init__(self, parent, ID, torrent, queue, panel, multitorrent, message1, message2 = None, destloc = None, destname = None, torfilename = None,
                 origtorname = None, dir = False, title = None, pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = wx.DEFAULT_DIALOG_STYLE):
        # May be called with a torrent, or with explicit destination location and destination name,
        # current torrent name and torrent file name (in this case torrent must be set to None)
        # This dialog is used :
        # 1 - to change the destination (location and name) of an existing torrent :
        #     In this case torrent will be set to the torrent to be changed
        # 2 - to choose a new destination for a new torrent because the destination is duplicate or badly named :
        #     In this case torrent will be set to None
        # Destination name length is only checked in case 1 ; in case 2, the global name checking will later check
        # paths including destination.
        self.queue = queue
        self.panel = panel
        self.multitorrent = multitorrent
        self.utility = parent.utility
        self.localize = self.utility.lang.get
        self.torrent = torrent
        if title is None:
            if self.multitorrent:
                title = self.localize('choosedowndestmulti') + message1
            else:
                title = self.localize('choosedowndest') + message1
        pre = wx.PreDialog()
        pre.Create(parent, ID, title, pos, size, style)
        self.this = pre.this

        if self.torrent is None:
            magnetname = currenttorname = origtorname
            torfilename = path.split(torfilename)[1]
            torfilename = torfilename[:torfilename.rfind('.torrent')]
        else:
            destloc = path.split(self.torrent.dest)[0]
            if self.multitorrent:
                destname = origtorname = magnetname = currenttorname = torfilename = ''
            else:
                if self.torrent.magnet == 1:
                    destname = origtorname = self.localize('notorrentinfo')
                elif self.torrent.magnet == 2:
                    destname = path.split(self.torrent.dest)[1]
                    origtorname = self.localize('notorrentinfo')
                else:
                    destname = path.split(self.torrent.dest)[1]
                    metainfo_file = open(self.torrent.src, 'rb')
                    metainfo = bdecode(metainfo_file.read(), sloppy = 1)
                    metainfo_file.close()
                    self.info = metainfo['info']
                    if self.info.has_key('name.utf-8'):
                        namekey = 'name.utf-8'
                    else:
                        namekey = 'name'
                    try:
                        origtorname = self.info[namekey].decode('utf_8')
                    except:
                        origtorname = self.info[namekey].decode(sysencoding)
                magnetname = self.torrent.magnetname
                currenttorname = self.torrent.name
                torfilename = path.split(self.torrent.src)[1]
                torfilename = torfilename[:torfilename.rfind('.torrent')]
        self.newdest = ''
        self.dir = dir

        outerbox = wx.BoxSizer(wx.VERTICAL)

        globalbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, ''), wx.VERTICAL)

        if message2:
            globalbox.Add(wx.StaticText(self, -1, message2), 0, wx.ALIGN_CENTER_VERTICAL | wx.TOP, 3)

        downdestlocbox = wx.StaticBoxSizer(wx.StaticBox(self, -1, self.localize('downdestloc')), wx.VERTICAL)
        downdestloc = wx.BoxSizer(wx.HORIZONTAL)
        self.downdestloctext = wx.TextCtrl(self, -1, destloc, (-1, -1), (400, -1))        
        downdestloc.Add(self.downdestloctext, 1)
        downdestlocbtn = wx.Button(self, -1, self.localize('browsebtn'), (-1, -1), (45, 20))
        downdestloc.Add(downdestlocbtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        downdestlocbox.Add(downdestloc, 0, wx.EXPAND | wx.TOP, 0)

        if dir or self.multitorrent:
            downdestnamehead = self.localize('downdirname')
        else:
            downdestnamehead = self.localize('downfilename')
        downdestnamebox = wx.StaticBoxSizer(wx.StaticBox(self, -1, downdestnamehead), wx.VERTICAL)
        downdestname = wx.BoxSizer(wx.HORIZONTAL)
        self.downdestnametext = wx.TextCtrl(self, -1, destname, (-1, -1), (400, -1))        
        downdestname.Add(self.downdestnametext, 1)
        downdestnamebtn = wx.Button(self, -1, self.localize('browsebtn'), (-1, -1), (45, 20))
        downdestname.Add(downdestnamebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        downdestnamebox.Add(downdestname, 0, wx.EXPAND | wx.TOP, 0)

        usenamefrombox = wx.StaticBoxSizer(wx.StaticBox(self, -1, self.localize('usenamefrom')), wx.VERTICAL)

        usenamefrombox.Add(wx.StaticText(self, -1, self.localize('origtorname')), 0, wx.TOP, -1)
        origtornamebox = wx.BoxSizer(wx.HORIZONTAL)
        self.origtornametext = wx.TextCtrl(self, -1, origtorname, (-1, -1), (-1, -1), wx.TE_READONLY)
        origtornamebox.Add(self.origtornametext, 1, wx.EXPAND)
        origtornamebtn = wx.Button(self, -1, self.localize('copybtn'), (-1, -1), (35, 20))
        origtornamebox.Add(origtornamebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        usenamefrombox.Add(origtornamebox, 1, wx.EXPAND | wx.TOP, 3)

        if self.torrent and self.torrent.magnetname:
            usenamefrombox.Add(wx.StaticText(self, -1, self.localize('magnetname')), 0, wx.TOP, 5)
            magnetnamebox = wx.BoxSizer(wx.HORIZONTAL)
            self.magnetnametext = wx.TextCtrl(self, -1, magnetname, (-1, -1), (-1, -1), wx.TE_READONLY)
            magnetnamebox.Add(self.magnetnametext, 1, wx.EXPAND)
            magnetnamebtn = wx.Button(self, -1, self.localize('copybtn'), (-1, -1), (35, 20))
            magnetnamebox.Add(magnetnamebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            usenamefrombox.Add(magnetnamebox, 1, wx.EXPAND | wx.TOP, 3)

        usenamefrombox.Add(wx.StaticText(self, -1, self.localize('currenttorname')), 0, wx.TOP, 5)
        currenttornamebox = wx.BoxSizer(wx.HORIZONTAL)
        self.currenttornametext = wx.TextCtrl(self, -1, currenttorname, (-1, -1), (-1, -1), wx.TE_READONLY)
        currenttornamebox.Add(self.currenttornametext, 1, wx.EXPAND)
        currenttornamebtn = wx.Button(self, -1, self.localize('copybtn'), (-1, -1), (35, 20))
        currenttornamebox.Add(currenttornamebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        usenamefrombox.Add(currenttornamebox, 1, wx.EXPAND | wx.TOP, 3)

        usenamefrombox.Add(wx.StaticText(self, -1, self.localize('torfilename')), 0, wx.TOP, 5)
        torfilenamebox = wx.BoxSizer(wx.HORIZONTAL)
        self.torfilenametext = wx.TextCtrl(self, -1, torfilename, (-1, -1), (-1, -1), wx.TE_READONLY)
        torfilenamebox.Add(self.torfilenametext, 1, wx.EXPAND)
        torfilenamebtn = wx.Button(self, -1, self.localize('copybtn'), (-1, -1), (35, 20))
        torfilenamebox.Add(torfilenamebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        usenamefrombox.Add(torfilenamebox, 1, wx.EXPAND | wx.TOP, 3)
        downdestnamebox.Add(usenamefrombox, 1, wx.EXPAND | wx.TOP, 5)

        self.rentorwithdest = wx.CheckBox(self, -1, self.localize('rentorwithdest'))
        downdestnamebox.Add(self.rentorwithdest, 0, wx.TOP, 3)

        self.movedata = wx.CheckBox(self, -1, self.localize('movedata'))

        globalbox.Add(downdestlocbox, 0, wx.TOP | wx.EXPAND, 5)
        globalbox.Add(downdestnamebox, 0, wx.TOP | wx.EXPAND, 5)
        globalbox.Add(self.movedata, 0, wx.TOP, 3)

        outerbox.Add(globalbox, 0, wx.TOP, -4)

        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        self.okbtn = wx.Button(self, -1, self.localize('ok'), size = (60, -1))
        self.cancelbtn = wx.Button(self, -1, self.localize('cancel'), size = (60, -1))
        buttonbox.Add(self.okbtn, 0, wx.ALL, 5)
        buttonbox.Add(self.cancelbtn, 0, wx.ALL, 5)

        outerbox.Add(buttonbox, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(outerbox)
        self.Fit()

        # Add events
        self.Bind(wx.EVT_BUTTON, self.onChangeLocation, downdestlocbtn)
        self.Bind(wx.EVT_BUTTON, self.onChangeName, downdestnamebtn)
        self.Bind(wx.EVT_BUTTON, self.onCopyCurName, currenttornamebtn)
        self.Bind(wx.EVT_BUTTON, self.onCopyOrigName, origtornamebtn)
        if self.torrent and self.torrent.magnetname:
            self.Bind(wx.EVT_BUTTON, self.onCopyMagnetName, magnetnamebtn)
        self.Bind(wx.EVT_BUTTON, self.onCopyTorFileName, torfilenamebtn)
        self.Bind(wx.EVT_BUTTON, self.onOK, self.okbtn)
        self.Bind(wx.EVT_BUTTON, self.onCancel, self.cancelbtn)
        self.Bind(wx.EVT_CLOSE, self.onCancel)
        self.downdestloctext.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        self.downdestnametext.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)

        if self.utility.abcparams['abcsetdestdlgx'] != '-1':
            self.MoveXY(int(self.utility.abcparams['abcsetdestdlgx']), int(self.utility.abcparams['abcsetdestdlgy']))

        if self.multitorrent or self.torrent and self.torrent.magnet == 1:
            # Multi-torrent or magnet : only down directory location will be available for change
            self.downdestnametext.Disable()
            downdestnamebtn.Disable()
            self.origtornametext.Disable()
            origtornamebtn.Disable()
            if self.torrent and self.torrent.magnetname:
                self.magnetnametext.Disable()
                magnetnamebtn.Disable()
            self.currenttornametext.Disable()
            currenttornamebtn.Disable()
            self.torfilenametext.Disable()
            torfilenamebtn.Disable()
            self.rentorwithdest.Disable()
            self.movedata.Disable()
        elif self.utility.abcparams['defrentorwithdest'] == "1":
            self.rentorwithdest.SetValue(True)

    def getPath(self):
        return self.newdest

    def getRenTorWithDest(self):
        return self.rentorwithdest.GetValue()

    def onChangeLocation(self, event):
        dl = wx.DirDialog(self, self.localize('choosenewdestloc'), self.downdestloctext.GetValue(),
                          style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        result = dl.ShowModal()
        if result != wx.ID_OK:
            dl.Destroy()
            return
        dest = dl.GetPath()
        dl.Destroy()
        self.downdestloctext.SetValue(dest)

    def onChangeName(self, event):
        if self.dir:
            dl = wx.DirDialog(self, self.localize('choosenewdirname'),
                              path.join(self.downdestloctext.GetValue(), self.downdestnametext.GetValue()),
                              style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        else:
            dl = wx.FileDialog(self, self.localize('choosenewfilename'), self.downdestloctext.GetValue(),
                               self.downdestnametext.GetValue(), '*.*', wx.SAVE)                
        result = dl.ShowModal()
        if result != wx.ID_OK:
            dl.Destroy()
            return
        dest = dl.GetPath()
        dl.Destroy()
        self.downdestloctext.SetValue(path.split(dest)[0])
        self.downdestnametext.SetValue(path.split(dest)[1])

    def onCopyOrigName(self, event):
        self.downdestnametext.SetValue(self.origtornametext.GetValue())

    def onCopyMagnetName(self, event):
        self.downdestnametext.SetValue(self.magnetnametext.GetValue())

    def onCopyCurName(self, event):
        self.downdestnametext.SetValue(self.currenttornametext.GetValue())

    def onCopyTorFileName(self, event):
        self.downdestnametext.SetValue(self.torfilenametext.GetValue())

    def onCancel(self, event = None):
        self.EndModal(wx.ID_CANCEL)

    def onOK(self, event):
        newdowndestloc = self.downdestloctext.GetValue()

        # Check if location name is valid in Windows
        if sys.platform == 'win32':
            # We erase the final '\' except for a path like 'X:\'
            if newdowndestloc and newdowndestloc[-1] == '\\' and (len(newdowndestloc) < 2 or newdowndestloc[-2] != ':'):
                newdowndestloc = newdowndestloc[:-1]
            newdowndestloc = newdowndestloc.rstrip(whitespace + '.')
            if not self.utility.checkWinPath(self, newdowndestloc):
                return

        selectedtorrents = []
        toolongdestpath = False
        if self.multitorrent or self.torrent and self.torrent.magnet == 1:
            # Just update the location for all selected torrents
            for index in self.panel.getSelected():
                torrent = self.queue.proctab[index]
                if torrent.abcengine or torrent.status == 'queue' or torrent.ismovingdata:
                    continue
                self.newdest = path.join(newdowndestloc, path.split(torrent.dest)[1])
                selectedtorrents.append((torrent, index, self.newdest))
                destlen = len(self.utility.completePath(self.newdest))
                if torrent.singlefile:
                    if destlen > 259:
                        toolongdestpath = True
                else:
                    # Check path length for each file of the torrent
                    for f in torrent.getFileNames():
                        if len(f) + destlen > 258 or len(path.split(f)[0]) + destlen > 246:
                            toolongdestpath = True
                            break
                    # Reset bit 4 in file name status
                    torrent.resetFileNameStatus(mask = 7)
                if toolongdestpath:
                    dlg = wx.MessageDialog(self, self.localize('torrentname') + torrent.name
                                           + '\n' + self.localize('errortoolongdestpath'),
                                           self.localize('abcokcerror'), wx.ICON_ERROR)
                    dlg.ShowModal()
                    dlg.Destroy()
                    return
        else:
            newdowndestname = self.downdestnametext.GetValue()
            # Check if file and folder names are valid in Windows
            if sys.platform == 'win32':
                newdowndestname = newdowndestname.rstrip(whitespace + '.')
                fixedname = self.utility.fixWindowsName(newdowndestname)
                if fixedname:
                    dlg = wx.MessageDialog(self, newdowndestname + '\n' + self.localize('errorinvalidwinname') + fixedname,
                                           self.localize('abcokcerror'), wx.ICON_ERROR)
                    dlg.ShowModal()
                    dlg.Destroy()
                    return

            self.newdest = path.join(newdowndestloc, newdowndestname)

            if self.torrent is not None:
                try:
                    index = self.queue.proctab.index(self.torrent)
                except:
                    # Torrent has been deleted from list
                    dlg = wx.MessageDialog(self, self.localize('errortorrentnolongerinlist'),
                                           self.localize('abcokcerror'), wx.ICON_ERROR)
                    dlg.ShowModal()
                    dlg.Destroy()
                    self.EndModal(wx.ID_CANCEL)
                destlen = len(self.utility.completePath(self.newdest))
                if self.torrent.singlefile:
                    if destlen > 259:
                        toolongdestpath = True
                else:
                    # Check path length for each file of the torrent
                    for f in self.torrent.getFileNames(filesinfo = self.info['files']):
                        if len(f) + destlen > 258 or len(path.split(f)[0]) + destlen > 246:
                            toolongdestpath = True
                            break
                    # Reset bit 4 in file name status
                    self.torrent.resetFileNameStatus(mask = 7)
                if toolongdestpath:
                    dlg = wx.MessageDialog(self, self.localize('errortoolongdestpath'),
                                           self.localize('abcokcerror'), wx.ICON_ERROR)
                    dlg.ShowModal()
                    dlg.Destroy()
                    return
                selectedtorrents.append((self.torrent, index, self.newdest))

        for t, i, nd in selectedtorrents:
            if t is not None and (not t.abcengine and t.status != 'queue'
                                  and t.status != 'standby' and not t.ismovingdata):
                destlowercase = self.utility.completePath(nd).lower()
                # Search for duplicate
                for t2 in self.queue.proctab:
                    if t2 != t and self.utility.completePath(t2.dest).lower() == destlowercase:
                        dlg = AddSameDestDialog(self, -1, self.localize('duplicatedestination'),
                                                t.name, t.src, False, loading = False,
                                                pos = self.queue.addtorrentwarningpos)
                        result = dlg.ShowModal()
                        self.queue.addtorrentwarningpos = dlg.GetPosition()
                        dlg.Destroy()
                        if result == 0:
                            return False
                        break

                # Update destination
                if self.rentorwithdest.GetValue():
                    # Update current torrent name field in dialog
                    self.currenttornametext.SetValue(newdowndestname)
                olddest = self.utility.completePath(t.dest)
                self.queue.changeProcDest(i, nd, self.rentorwithdest.GetValue())
                if self.movedata.GetValue():
                    self.queue.dataMovingToNewDestThread(t, olddest)

        self.EndModal(wx.ID_OK)
