##############################################################################
# Module : abcdetailframe.py
# Author :
# Date   :
# Updated and modified for ABC_OKC : Old King Cole
#
# Description : Torrent advanced details window
#
##############################################################################
import sys, wx, re, binascii
from os.path import join
from time import clock

import abcmetainfoframe


class ABCDetailFrame(wx.Frame):
    def __init__(self, parent, ID, title, utility, torrentsrc, index, position, size, centeratpointer = False):        
        if position is None or position < 0:
            pos = wx.DefaultPosition
        else:
            pos = position
        wx.Frame.__init__(self, parent, ID, title, pos, size)

        self.parent = parent
        self.utility = utility
        self.abcparams = self.utility.abcparams
        self.localize = self.utility.lang.get
        self.guiman = self.parent.guiman
        self.resettingframe = False
        self.index = index
        self.torrent = self.parent.queue.proctab[index]
        self.width = size[0]
        # To save pos and size for an opened window currently browsed in another one
        self.wintorestore = None

        if sys.platform == 'win32':
            self.icon = wx.Icon(join(self.utility.abcpath, 'icon_abc.ico'), wx.BITMAP_TYPE_ICO)
            self.SetIcon(self.icon)

        self.panel = wx.Panel(self, -1)
        colsizer = wx.FlexGridSizer(cols = 1, vgap = 3)
        colsizer.AddGrowableCol(0)
        colsizer.AddGrowableRow(1)

        toppanel = wx.Panel(self.panel, -1)

        topsizer = wx.BoxSizer(wx.VERTICAL)

        self.abouttitle = wx.StaticText(toppanel, -1, '')
        self.abouttitle.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.NORMAL, False))
        topsizer.Add(self.abouttitle, 0, wx.EXPAND)

        detailsizer = wx.FlexGridSizer(cols = 4, vgap = 3, hgap = 5)
        detailsizer.AddGrowableCol(0, 1)
        detailsizer.AddGrowableCol(1, 1)
        detailsizer.AddGrowableCol(2, 1)
        detailsizer.AddGrowableCol(3, 1)
        detailsizer1 = wx.FlexGridSizer(cols = 2, vgap = 3, hgap = 2)
        detailsizer2 = wx.FlexGridSizer(cols = 2, vgap = 3, hgap = 2)
        detailsizer3 = wx.FlexGridSizer(cols = 2, vgap = 3, hgap = 2)
        detailsizer4 = wx.FlexGridSizer(cols = 2, vgap = 3, hgap = 2)

        # # Seeds
        self.seedtitle = wx.StaticText(toppanel, -1, self.localize('dnumconnectedseed'))
        self.numseed = wx.StaticText(toppanel, -1, '')        
        detailsizer1.Add(self.seedtitle)
        detailsizer1.Add(self.numseed)

        # # Peers
        peertitle = wx.StaticText(toppanel, -1, self.localize('dnumconnectedpeer'))
        self.numpeer = wx.StaticText(toppanel, -1, '')        
        detailsizer2.Add(peertitle)
        detailsizer2.Add(self.numpeer)

        # # Seeing Copies
        copytitle = wx.StaticText(toppanel, -1, self.localize('dseeingcopies'))
        self.numcopy = wx.StaticText(toppanel, -1, '')       
        detailsizer3.Add(copytitle)
        detailsizer3.Add(self.numcopy)

        # Avg peer
        avgprogresstitle = wx.StaticText(toppanel, -1, self.localize('davgpeerprogress'))
        self.avgprogress = wx.StaticText(toppanel, -1, '')        
        detailsizer4.Add(avgprogresstitle)
        detailsizer4.Add(self.avgprogress)

        # Download Size
        downsizetitle = wx.StaticText(toppanel, -1, self.localize('ddownloadedsize'))
        self.downsize = wx.StaticText(toppanel, -1, '')        
        detailsizer1.Add(downsizetitle)
        detailsizer1.Add(self.downsize)

        # Upload Size
        upsizetitle = wx.StaticText(toppanel, -1, self.localize('duploadedsize'))
        self.upsize = wx.StaticText(toppanel, -1, '')        
        detailsizer2.Add(upsizetitle)
        detailsizer2.Add(self.upsize)

        # Total Speed
        totalspeedtitle = wx.StaticText(toppanel, -1, self.localize('dtotalspeed'))
        self.totalspeed = wx.StaticText(toppanel, -1, '')        
        detailsizer3.Add(totalspeedtitle)
        detailsizer3.Add(self.totalspeed)

        # Total Progress
        totalprogresstitle = wx.StaticText(toppanel, -1, self.localize('totalprogressedetail'))
        self.totalprogress = wx.StaticText(toppanel, -1, '')        
        detailsizer4.Add(totalprogresstitle)
        detailsizer4.Add(self.totalprogress)

        detailsizer.Add(detailsizer1)
        detailsizer.Add(detailsizer2)
        detailsizer.Add(detailsizer3)
        detailsizer.Add(detailsizer4)

        topsizer.Add(detailsizer, 0, wx.EXPAND | wx.TOP, 2)

        toppanel.SetSizer(topsizer)
        toppanel.SetAutoLayout(True)
        colsizer.Add(toppanel, 0, wx.EXPAND | wx.TOP, -2)

        self.spewlist = wx.ListCtrl(self.panel, -1, style = wx.LC_REPORT | wx.LC_HRULES | wx.LC_VRULES)

        colsizer.Add(self.spewlist, 0, wx.EXPAND)

        bottompanel = wx.Panel(self.panel, -1)

        currentstatsizer = wx.BoxSizer(wx.VERTICAL)

        currentstatsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        statusbox = wx.BoxSizer(wx.HORIZONTAL)
        statustitle = wx.StaticText(bottompanel, -1, self.localize('statusdetwin'))
        self.status = wx.StaticText(bottompanel, -1, '', size = (75, -1))
        statusbox.Add(statustitle)
        statusbox.Add(self.status, 0, wx.LEFT, 5)
        self.colour = wx.Window(bottompanel, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colour.SetBackgroundColour(self.utility.colnoeng)
        statusbox.Add(self.colour, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        currentstatsizer1.Add(statusbox)
        self.storagestats1 = wx.StaticText(bottompanel, -1, '', style = wx.ALIGN_RIGHT | wx.ST_NO_AUTORESIZE)
        currentstatsizer1.Add(self.storagestats1, 1)

        currentstatsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        ratesbox = wx.BoxSizer(wx.HORIZONTAL)
        downratetitle = wx.StaticText(bottompanel, -1, self.localize('downratedetwin'))
        self.downrate = wx.StaticText(bottompanel, -1, '', size = (60, -1))
        ratesbox.Add(downratetitle)
        ratesbox.Add(self.downrate, 0, wx.LEFT, 5)
        upratetitle = wx.StaticText(bottompanel, -1, self.localize('upratedetwin'))
        self.uprate = wx.StaticText(bottompanel, -1, '', size = (60, -1))
        ratesbox.Add(upratetitle, 0, wx.LEFT, 10)
        ratesbox.Add(self.uprate, 0, wx.LEFT, 5)
        currentstatsizer2.Add(ratesbox, 0)
        self.storagestats2 = wx.StaticText(bottompanel, -1, '', style = wx.ALIGN_RIGHT | wx.ST_NO_AUTORESIZE)
        currentstatsizer2.Add(self.storagestats2, 1)

        currentstatsizer.Add(currentstatsizer1, 0, wx.EXPAND)
        currentstatsizer.Add(currentstatsizer2, 0, wx.EXPAND | wx.TOP, 3)

        bottompanel.SetSizer(currentstatsizer)
        bottompanel.SetAutoLayout(True)
        colsizer.Add(bottompanel, 0, wx.EXPAND)

        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)

        scrapebut = wx.Button(self.panel, -1, self.localize('updateseedpeer'))
        buttonsizer.Add(scrapebut)

        locatebut = wx.Button(self.panel, -1, self.localize('locate'), size = (55, -1))
        buttonsizer.Add(locatebut, 0, wx.LEFT, 10)

        torrentparambut = wx.Button(self.panel, -1, self.localize('torrentparam'))
        buttonsizer.Add(torrentparambut, 0, wx.LEFT, 10)

        self.detailsbut = wx.Button(self.panel, -1, self.localize('details'))
        buttonsizer.Add(self.detailsbut, 0, wx.LEFT, 10)

        closebut = wx.Button(self.panel, -1, self.localize('close'), size = (55, -1))
        buttonsizer.Add(closebut, 0, wx.LEFT, 10)

        colsizer.Add(buttonsizer, 0, wx.ALIGN_CENTER | wx.TOP, 3)

        col = 0
        for colinfo in self.utility.advdetcolumns:
            self.spewlist.InsertColumn(col, colinfo[0], format = colinfo[2], width = colinfo[1])
            col += 1

        border = wx.BoxSizer(wx.HORIZONTAL)
        border.Add(colsizer, 1, wx.EXPAND | wx.ALL, 3)
        self.panel.SetSizer(border)
        self.panel.SetAutoLayout(True)

        self.spewlist.Bind(wx.EVT_LIST_COL_END_DRAG, self.onResizeColumn)
        self.spewlist.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.onColRightClick)
        self.Bind(wx.EVT_SIZE, self.onSizeAdvDet)
        self.Bind(wx.EVT_BUTTON, self.onLocate, locatebut)
        self.Bind(wx.EVT_BUTTON, self.onLocalSetting, torrentparambut)
        self.Bind(wx.EVT_BUTTON, self.onDetails, self.detailsbut)
        self.Bind(wx.EVT_BUTTON, self.getScrape, scrapebut)
        self.Bind(wx.EVT_CLOSE, self.killAdv)
        self.Bind(wx.EVT_BUTTON, self.killAdv, closebut)
        self.abouttitle.Bind(wx.EVT_LEFT_DOWN, self.focusOnTitleAdv)
        self.abouttitle.Bind(wx.EVT_MOUSEWHEEL, self.onMouseWheelAdv)
        self.panel.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        toppanel.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        bottompanel.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.abouttitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.seedtitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.numseed.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        peertitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.numpeer.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        copytitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.numcopy.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        avgprogresstitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.avgprogress.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        downsizetitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.downsize.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        upsizetitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.upsize.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        totalspeedtitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.totalspeed.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        totalprogresstitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.totalprogress.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        statustitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.status.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        downratetitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.downrate.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        upratetitle.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.uprate.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.storagestats1.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.storagestats2.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)
        self.colour.Bind(wx.EVT_RIGHT_UP, self.onRightUpAdv)

        self.displayInfo()

        if position == -1:
            self.Centre()
        elif position == -2:
            self.Move(self.utility.getPosCenteredOnWin(size, self.torrent.infowin))
        elif position == -3:
            self.Move(self.utility.getPosCenteredOnWin(size, self.torrent.infowin, aligntop = True))
        elif centeratpointer:
            self.Move(pos - wx.Point(self.GetSizeTuple()[0] / 2, 10))

        self.resettingframe = True
        self.Show()
        self.resettingframe = False

    def displayInfo(self):
        self.panel.Freeze()
        self.SetTitle(self.localize('torrentadvdetail') + self.torrent.name)
        self.abouttitle.SetLabel(self.torrent.name.replace('&', '&&'))
        if self.torrent.abcengine:
            self.spewlist.DeleteAllItems()
            self.torrent.abcengine.updateDetailWindowNow(self)
        else:
            self.init()
            self.updateSize()
            self.totalprogress.SetLabel(self.torrent.formatedProgress())
            if self.torrent.ismovingdata:
                self.status.SetLabel(self.localize('moving'))
            else:
                self.status.SetLabel(self.localize(self.torrent.status))
            self.colour.SetBackgroundColour(self.utility.colnoeng)
            self.colour.Refresh()

        self.resetFrame()

    def resetFrame(self):
        self.resettingframe = True
        self.panel.SetSize(self.GetClientSize())
        self.panel.Layout()
        self.sizeTitle()
        self.panel.Thaw()
        self.resettingframe = False

    def focusOnTitleAdv(self, event):
        self.abouttitle.SetFocus()

    def onSizeAdvDet(self, event):
        if self.resettingframe:
            event.Skip()
            return
        oldw = self.width
        self.width = event.GetSize()[0]
        if self.width != oldw:
            self.panel.Freeze()
            self.resetFrame()
        else:
            event.Skip()

    def setSize(self, size):
        self.SetSize(size)

    def sizeTitle(self):
        if self.abouttitle.GetTextExtent(self.torrent.name)[0] > self.width - 14:
            self.abouttitle.SetSize((5000, -1))
            self.abouttitle.SetWindowStyle(wx.ALIGN_LEFT)
            self.abouttitle.SetToolTipString(self.torrent.name)
        else:
            self.abouttitle.SetSize((self.width - 14, -1))
            self.abouttitle.SetWindowStyle(wx.ALIGN_CENTER)
            self.abouttitle.SetToolTipString('')

    def onMouseWheelAdv(self, event):
        oldindex = self.index
        if event.m_wheelRotation > 0:
            if self.index > 0:
                self.index -= 1
            else:
                return
        elif self.index < self.parent.list.GetItemCount() - 1:
            self.index += 1
        else:
            return

        oldtorrent = self.torrent
        self.torrent = self.parent.queue.proctab[self.index]

        if self.wintorestore is not None:
            # Restore window for old torrent
            oldtorrent.detailwin = ABCDetailFrame(self.parent, -1, '', self.utility, oldtorrent.src, oldindex,
                                                  self.wintorestore[0], self.wintorestore[1], centeratpointer = False)
            if oldtorrent.abcengine:
                oldtorrent.abcengine.detailwin = oldtorrent.detailwin
            self.abouttitle.SetFocus()
        else:
            oldtorrent.detailwin = None
            if oldtorrent.abcengine:
                oldtorrent.abcengine.detailwin = None

        if self.torrent.detailwin is not None:
            # Kill window for new torrent and save position and size if not hidden for later restoration
            if self.torrent.detailwin.IsShown():
                if self.torrent.detailwin.IsIconized() or self.torrent.detailwin.IsMaximized():
                    self.wintorestore = ((-1, -1), (self.guiman.getDetailWidth(), self.guiman.getDetailHeight()))
                else:
                    self.wintorestore = (self.torrent.detailwin.GetPosition(), self.torrent.detailwin.GetSize())
            else:
                self.wintorestore = None
            self.torrent.detailwin.killAdv()
        else:
            self.wintorestore = None

        self.torrent.detailwin = self
        if self.torrent.abcengine:
            self.torrent.abcengine.detailwin = self
        self.displayInfo()

        # If the old selection in list matched the displayed info, change the selection in list
        if self.parent.list.GetSelectedItemCount() == 1 and self.parent.getSelected()[0] == oldindex:
            self.parent.list.SetItemState(oldindex, 0, wx.LIST_STATE_SELECTED)
            self.parent.list.Select(self.index)
            if event.m_wheelRotation > 0:
                if self.index > 0:
                    self.parent.list.EnsureVisible(self.index - 1)
            elif self.index < self.parent.list.GetItemCount() - 1:
                self.parent.list.EnsureVisible(self.index + 1)
            self.parent.list.EnsureVisible(self.index)

        if oldtorrent.infowin is not None and not oldtorrent.infowin.IsShown():
            oldtorrent.infowin.killInfo()

    def updateTorrentName(self):
        self.SetTitle(self.localize('torrentadvdetail') + self.torrent.name)
        self.abouttitle.SetLabel(self.torrent.name.replace('&', '&&'))
        self.sizeTitle()

    def updateSize(self):
        # Update downloaded and uploaded size display
        if self.abcparams['displayunitsize'] == '1':
            self.downsize.SetLabel(self.utility.formatedSize(self.torrent.downsize))
            self.upsize.SetLabel(self.utility.formatedSize(self.torrent.upsize))
        else:
            self.downsize.SetLabel(self.utility.formatedSize(self.torrent.downsize) + ' ' + self.utility.mb)
            self.upsize.SetLabel(self.utility.formatedSize(self.torrent.upsize) + ' ' + self.utility.mb)

    def onRightUpAdv(self, event):
        self.parent.makePopupMenu(self, False)
        self.PopupMenu(self.parent.popupmenu, self.panel.ScreenToClient(event.GetEventObject().ClientToScreen(event.GetPosition())))

    def onLocate(self, event):
        # Unselect all torrents in main ABC list
        for index in self.parent.getSelected():
            self.parent.list.SetItemState(index, 0, wx.LIST_STATE_SELECTED)
        self.parent.list.EnsureVisible(self.index)
        self.parent.list.Select(self.index)

    def onLocalSetting(self, event):
        self.parent.windowparent = self
        self.parent.commandfromlist = False
        self.parent.onLocalSetting()

    def onDetails(self, event):
        if self.torrent.infowin == None:
            centerdetoneachother = self.abcparams['centerdetoneachother']
            if centerdetoneachother == '0':
                infowinpos = self.GetPosition()
            elif centerdetoneachother == '1':
                infowinpos = -2
            else:
                infowinpos = -3
            self.torrent.infowin = abcmetainfoframe.ABCMetaInfoFrame(self.parent, -1, '', self.utility,
                                                                     self.torrent.src, self.index,
                                                                     infowinpos,
                                                                     self.guiman.getMetaSize())
            if self.torrent.abcengine:
                self.torrent.abcengine.infowin = self.torrent.infowin
        elif self.torrent.infowin.IsShown():
            if self.torrent.infowin.IsIconized():
                self.torrent.infowin.Iconize(False)
            self.torrent.infowin.Raise()
        else:
            centerdetoneachother = self.abcparams['centerdetoneachother']
            if centerdetoneachother == '0':
                infowinpos = self.GetPosition()
            elif centerdetoneachother == '1':
                infowinpos = self.utility.getPosCenteredOnWin(self.torrent.infowin.GetSize(), self)
            else:
                infowinpos = self.utility.getPosCenteredOnWin(self.torrent.infowin.GetSize(), self, aligntop = True)
            self.torrent.infowin.Move(infowinpos)
            self.torrent.infowin.Show()
        self.torrent.detailwin.Hide()
        x, y, w, h = self.torrent.infowin.advdetailsbut.GetRect()
        self.torrent.infowin.WarpPointer(x + w / 2, y + h / 2)

    def getScrape(self, event):
        ctrl = wx.GetKeyState(wx.WXK_CONTROL)
        if self.torrent.scraping or self.torrent.lastgetscrape and clock() - self.torrent.lastgetscrape < 60:
            dlg = wx.MessageDialog(self, self.localize('warningscrape2') + self.localize('warningscrape3'),
                                   self.localize('abcokcwarning'), wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
            return
        self.parent.queue.getScrapeData(self.torrent, self.index, ctrlscrape = ctrl)

    def killAdv(self, event = None):
        if self.torrent.infowin is not None and not self.torrent.infowin.IsShown():
            self.torrent.infowin.killInfo()
        self.parent.queue.deleteDetailWin(self.index)
        self.Destroy()

    def init(self):
        self.numseed.SetLabel('0 (' + self.torrent.seed + ')')
        self.numpeer.SetLabel('0 (' + self.torrent.peer + ')')
        self.numcopy.SetLabel(self.torrent.numcopy)
        self.avgprogress.SetLabel('0.0%')
        self.totalspeed.SetLabel('0.0 ' + self.utility.kbps)
        for i in xrange(self.spewlist.GetItemCount()):
            self.spewlist.DeleteItem(0)
        self.downrate.SetLabel('0.0 ' + self.utility.kbps)
        self.uprate.SetLabel('0.0 ' + self.utility.kbps)
        self.storagestats1.SetLabel('')
        self.storagestats2.SetLabel('')

    def pause(self):
        self.numseed.SetLabel('0 (' + self.torrent.seed + ')')
        self.numpeer.SetLabel('0 (' + self.torrent.peer + ')')
        self.avgprogress.SetLabel("0.0%")
        self.totalspeed.SetLabel('0.0 ' + self.utility.kbps)
        for i in xrange(self.spewlist.GetItemCount()):
            self.spewlist.DeleteItem(0)
        self.downrate.SetLabel('0.0 ' + self.utility.kbps)
        self.uprate.SetLabel('0.0 ' + self.utility.kbps)
        self.storagestats1.SetLabel('')
        self.storagestats2.SetLabel('')

    def updateColumnWidth(self, col):
        # Save a column width from the spew list
        newcolwidth = self.spewlist.GetColumnWidth(col)
        self.utility.advdetcolumns[col][1] = newcolwidth
        self.abcparams[self.utility.advdetcolinfo[col][1]] = str(newcolwidth)

    def onResizeColumn(self, event):
        event.Skip()
        wx.CallAfter(self.updateColumnWidth, event.GetColumn())

    def onColRightClick(self, event):
        col = 0
        for colinfo in self.utility.advdetcolinfo:
            self.spewlist.SetColumnWidth(col, colinfo[3])
            self.utility.advdetcolumns[col][1] = colinfo[3]
            self.abcparams[colinfo[1]] = str(colinfo[3])
            col += 1

    def setAllColumnWidths(self):
        col = 0
        for colinfo in self.utility.advdetcolumns:
            self.spewlist.SetColumnWidth(col, colinfo[1])
            col += 1
